#FILE: combine_output_files.pl
#AUTH: Paul Stothard (stothard@ualberta.ca)
#DATE: Nov. 26, 2007
#VERS: 1.0

use warnings;
use strict;
use Data::Dumper;
use Getopt::Long;

#initialize options
my %options = (
    infile   => undef,
    outfile  => undef,
    header   => undef,
    comments => undef,
    verbose  => undef
);

#get command line options
GetOptions(
    'i=s' => \$options{infile},
    'o=s' => \$options{outfile},
    'h'   => \$options{header},
    'c'   => \$options{comments},
    'v'   => \$options{verbose}
);

#check for options
if (   !( defined( $options{infile} ) )
    or !( defined( $options{outfile} ) ) )
{
    die( print_usage() . "\n" );
}

my @parts = ();

#file.txt_part_aa, file.txt_part_ab, file.txt_part_ac
if ( $options{infile} =~ m/([^\/]+_part_)([a-zA-Z]+?)$/ ) {
    my $base      = $1;
    my $extension = $2;
    my $dir       = $options{infile};
    $dir =~ s/\Q$base$extension\E$//;
    opendir( my $DIR, $dir ) or die("Can't opendir $dir: $!");
    while ( my $file = readdir($DIR) ) {
        if ( $file =~ m/\Q$base\E[a-zA-Z]+?$/ ) {
            push( @parts, "$dir$file" );
        }
    }
    closedir($DIR);
    @parts = map { $_->[1] }
        sort { $a->[0] cmp $b->[0] }
        map { [ part_value($_), $_ ] } @parts;

    sub part_value {
        my $value = shift();
        if ( $value =~ m/_part_([a-zA-Z]+?)$/ ) {
            return $1;
        }
        else {
            die("Unable to parse $value");
        }
    }
}

#file_1.txt, file_2.txt, file_3.txt
if ( $options{infile} =~ m/([^\/]+_)(\d+)(\..+?)$/ ) {
    my $base      = $1;
    my $extension = $2;
    my $suffix    = $3;
    my $dir       = $options{infile};
    $dir =~ s/\Q$base$extension$suffix\E$//;
    opendir( my $DIR, $dir ) or die("Can't opendir $dir: $!");
    while ( my $file = readdir($DIR) ) {
        if ( $file =~ m/\Q$base\E\d+\Q$suffix\E$/ ) {
            push( @parts, "$dir$file" );
        }
    }
    closedir($DIR);
    @parts = map { $_->[1] }
        sort { $a->[0] <=> $b->[0] }
        map { [ number_value($_), $_ ] } @parts;

    sub number_value {
        my $value = shift();
        if ( $value =~ m/[^\/]+_(\d+)\..+?$/ ) {
            return $1;
        }
        else {
            die("Unable to parse $value");
        }
    }
}

#file_1, file_2, file_3
if ( $options{infile} =~ m/([^\/]+_)(\d+?)$/ ) {
    my $base      = $1;
    my $extension = $2;
    my $dir       = $options{infile};
    $dir =~ s/\Q$base$extension\E$//;
    opendir( my $DIR, $dir ) or die("Can't opendir $dir: $!");
    while ( my $file = readdir($DIR) ) {
        if ( $file =~ m/\Q$base\E\d+?$/ ) {
            push( @parts, "$dir$file" );
        }
    }
    closedir($DIR);
    @parts = map { $_->[1] }
        sort { $a->[0] <=> $b->[0] }
        map { [ number_suffix_value($_), $_ ] } @parts;

    sub number_suffix_value {
        my $value = shift();
        if ( $value =~ m/[^\/]+_(\d+?)$/ ) {
            return $1;
        }
        else {
            die("Unable to parse $value");
        }
    }
}

if ( $options{verbose} ) {
    print scalar(@parts)
        . " member files found in group containing $options{infile}\n";
#    print "Combining files in following order: "
#        . join( ', ', @parts ) . "\n";
}

my $comments = 0;
my $header   = 0;
my $other    = 0;
open( my $OUTFILE, '>', $options{outfile} )
    or die("Cannot open file '$options{outfile}': $!");
for ( my $i = 0; $i < scalar(@parts); $i++ ) {
    open( my $INFILE, $parts[$i] )
        or die("Cannot open file '$parts[$i]': $!");
    my $header_passed = 0;
    while ( my $line = <$INFILE> ) {
        if ( $line =~ m/^\s*$/ ) {
            next;
        }
        if ( $line =~ m/^\#/ ) {
            if ( ( $i == 0 ) && ( $options{comments} ) ) {
                print( $OUTFILE $line );
                $comments++;
            }
            next;
        }
        if ( $options{header} ) {
            if ( $i == 0 ) {
                if ( !$header_passed ) {
                    print( $OUTFILE $line );
                    $header++;
                    $header_passed = 1;
                    next;
                }
            }
            else {
                if ( !$header_passed ) {
                    $header_passed = 1;
                    next;
                }
            }
        }
        print( $OUTFILE $line );
        $other++;
    }
    close($INFILE) or die("Cannot close file: $!");
}

close($OUTFILE) or die("Cannot close file: $!");

if ( $options{verbose} ) {
#    print "Comments written to '$options{outfile}': $comments\n";
#    print "Header lines written to '$options{outfile}': $header\n";
    print "Data lines written to '$options{outfile}': $other\n";
}

sub print_usage {
    print <<BLOCK;
USAGE: perl combine_output_files.pl [-arguments]
 -i [FILE]     : one of the files in the group to be combined
                 (Required).
 -o [FILE]     : output file (Required).
 -h            : first line of each file is a header line.
 -c            : comments in first file should be written to output
                 file.
 -v            : verbose mode.
 perl combine_output_files.pl -i ./some_output.txt_part_aa -c -h \
-o some_output_combined.txt
BLOCK
}
